@@ -216,10 +216,19 @@ impl<'db, DB: HirDatabase> Semantics<'db, DB> {
         self.imp.type_of_expr(expr)
     }
 
+    /// Returns true in case a coercion happened.
+    pub fn type_of_expr_with_coercion(&self, expr: &ast::Expr) -> Option<(Type, bool)> {
+        self.imp.type_of_expr_with_coercion(expr)
+    }
+
     pub fn type_of_pat(&self, pat: &ast::Pat) -> Option<Type> {
         self.imp.type_of_pat(pat)
     }
 
+    pub fn type_of_pat_with_coercion(&self, expr: &ast::Pat) -> Option<Type> {
+        self.imp.type_of_pat_with_coercion(expr)
+    }
+
     pub fn type_of_self(&self, param: &ast::SelfParam) -> Option<Type> {
         self.imp.type_of_self(param)
     }
@@ -560,10 +569,18 @@ impl<'db> SemanticsImpl<'db> {
         self.analyze(expr.syntax()).type_of_expr(self.db, expr)
     }
 
+    fn type_of_expr_with_coercion(&self, expr: &ast::Expr) -> Option<(Type, bool)> {
+        self.analyze(expr.syntax()).type_of_expr_with_coercion(self.db, expr)
+    }
+
     fn type_of_pat(&self, pat: &ast::Pat) -> Option<Type> {
         self.analyze(pat.syntax()).type_of_pat(self.db, pat)
     }
 
+    fn type_of_pat_with_coercion(&self, pat: &ast::Pat) -> Option<Type> {
+        self.analyze(pat.syntax()).type_of_pat_with_coercion(self.db, pat)
+    }
+
     fn type_of_self(&self, param: &ast::SelfParam) -> Option<Type> {
         self.analyze(param.syntax()).type_of_self(self.db, param)
     }
